//
//  PTPrinter.h
//
//  Created by midmirror on 15/12/25.
//  Copyright © 2015年 midmirror. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreBluetooth/CoreBluetooth.h>
#import "DispatcherDefine.h"
#import "PTPrinter.h"

/** 当前通讯方式 */
typedef NS_ENUM(NSInteger, PTDispatchMode) {
    
    PTDispatchModeUnconnect  = 0,
    PTDispatchModeBLE        = 1,
    PTDispatchModeWiFi       = 2,
};

@class PTPrinter;

/** 发现新蓝牙 */
typedef void(^ScanPrinterBlock)(PTPrinter *printer);
typedef void(^ScanPrintersBlock)(NSDictionary *printerDict);

/** 连接成功、失败 */
typedef void(^ConnectSuccessBlock)();
typedef void(^ConnectFailureBlock)();

/** 可以发送、发送成功、发送失败、接收到返回值 */
typedef void(^SendSuccessBlock)();
typedef void(^SendFailureBlock)();
typedef void(^SendProgressBlock)(NSNumber *progress);
typedef void(^ReceiveDataBlock)(NSData *data);
typedef void(^PrinterStatusBlock)(NSString *status);
typedef void(^RSSIBlock)(NSNumber *RSSI);

/** 断开 */
typedef void(^UnconnectBlock)();

/**
 *  主要功能：实现和打印机的 BLE 和 WiFi通讯
 */
@interface PTDispatcher : NSObject

singletonH(PTDispatcher)

@property(strong,nonatomic,readwrite) PTPrinter *printerConnected;
@property(assign,nonatomic,readwrite) PTDispatchMode mode;

@property(copy,nonatomic,readwrite) PrinterStatusBlock   printerStatusBlock;            //发送结束，且收到返回值时执行的 Block
@property(copy,nonatomic,readwrite) SendSuccessBlock     sendSuccessBlock;        //发送结束时的 Block
@property(copy,nonatomic,readwrite) SendFailureBlock     sendFailureBlock;
@property(copy,nonatomic,readwrite) SendProgressBlock    sendProgressBlock;
@property(copy,nonatomic,readwrite) ReceiveDataBlock     receiveDataBlock;

@property(copy,nonatomic,readwrite) ScanPrinterBlock     findBluetoothBlock;
@property(copy,nonatomic,readwrite) ScanPrintersBlock    findBluetoothAllBlock;
@property(copy,nonatomic,readwrite) ScanPrintersBlock    findWiFiAllBlock;

@property(copy,nonatomic,readwrite) ConnectSuccessBlock  connectSuccessBlock;
@property(copy,nonatomic,readwrite) ConnectFailureBlock  connectFailureBlock;
@property(copy,nonatomic,readwrite) UnconnectBlock       unconnectBlock;
@property(copy,nonatomic,readwrite) RSSIBlock            readRSSIBlock;

/** 发送数据 */
- (void)sendData:(NSData *)data;

#pragma mark PTBluetooth

/** 扫描蓝牙 */
- (void)scanBluetooth:(ScanPrinterBlock)bluetoothBlock;         // 发现新蓝牙（同一个蓝牙如果状态更新会被重复发现），仅用于快速连接
- (void)scanBluetoothAll:(ScanPrintersBlock)bluetoothAllBlock;  // 发现所有的蓝牙（每3秒更新一次所有蓝牙状态），用于扫描连接

/** 扫描 WiFi */
- (void)scanWiFi:(ScanPrintersBlock)wifiAllBlock;

/** 停止扫描蓝牙 */
- (void)stopScanBluetooth;

/** 连接打印机 */
- (void)connectPrinter:(PTPrinter *)printer;
/** 断开打印机 */
- (void)disConnectPrinter;

/** 更新打印机状态（当已连接）*/
- (void)whenReadRSSI:(RSSIBlock)readRSSIBlock;

/** 当连接成功时 */
- (void)whenConnectSuccess:(ConnectSuccessBlock)connectSuccessBlock;

/** 当连接失败时 */
- (void)whenConnectFailure:(ConnectFailureBlock)connectFailureBlock;

/** 当断开连接时 */
- (void)whenUnconnect:(UnconnectBlock)unconnectBlock;

/** 发送数据 */
- (void)whenSendSuccess:(SendSuccessBlock)sendSuccessBlock; // 成功
- (void)whenSendFailure:(SendFailureBlock)sendFailureBlock; // 失败
- (void)whenSendProgressUpdate:(SendProgressBlock)sendProgressBlock;    // 发送进度
- (void)whenReceiveData:(ReceiveDataBlock)receiveDataBlock;
- (void)whenPrinterStatusUpdate:(PrinterStatusBlock)printerStatusBlock;         //  获取的打印机状态
@end
