//
//  PTHomeVC.m
//
//  Created by midmirror on 16/1/27.
//  Copyright © 2016年 midmirror. All rights reserved.
//

#import "PTCommandVC.h"
#import "Masonry.h"
#import "PTBarcodeVC.h"
#import "PTTextVC.h"
#import "PTPageVC.h"
#import "PTConnectVC.h"
#import "PTTestESC.h"
#import "PTDisconnectVC.h"
#import "WirelessPrinter-Swift.h"
#import "PTIcon.h"

@interface PTCommandVC ()

@property(strong,nonatomic,readwrite) FlatMainView *flatView;
@property(strong,nonatomic,readwrite) NSArray *btnArray;
@property(strong,nonatomic,readwrite) NSDictionary *tableRowDic;
@property(strong,nonatomic,readwrite) UILabel *titleLabel;
@property(assign,nonatomic,readwrite) CGFloat buttonWidth;
@property(assign,nonatomic,readwrite) CGFloat buttonHeight;
@property(assign,nonatomic,readwrite) NSInteger selectedSegmentIndex;
@property(strong,nonatomic,readwrite) NSString *currentPrinterName;

@property(strong,nonatomic,readwrite) UITableView *tableView;

@property(strong,nonatomic,readwrite) NSArray *titleArray;
@property(strong,nonatomic,readwrite) NSArray *detailArray;
@property(strong,nonatomic,readwrite) NSArray *iconArray;
@property(strong,nonatomic,readwrite) NSString *themeColor;
@property(strong,nonatomic,readwrite) PTDisconnectVC *disconnectVC;

@end

@implementation PTCommandVC

#pragma mark - Life Cycle

- (id)init {
    
    if (self = [super init]) {
        self.title = NSLocalizedString(@"Command", nil);
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    _tableKit = [FlatTableKit share];
    
    [self initSettings];
    [self initScreentFitDataSource];
    [self initDataSourceESC];
    
    [self initFlatMainView];
    [self initTableView];
    [self initTitleLabel];
    [self initSegment];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:YES];
    
    [self setChangeView];
    [self startShimmering];
    
    self.navigationController.navigationBar.translucent = NO;
    
    //隐藏导航栏下面的线条
    [self.navigationController.navigationBar setBackgroundImage:[[UIImage alloc] init] forBarMetrics:UIBarMetricsDefault];
    self.navigationController.navigationBar.shadowImage = [[UIImage alloc] init];
    
    [self addDisconnectVC];
    
    [[PTDispatcher share] whenUnconnect:^{
       
        self.navigationController.navigationBar.barTintColor = FLAT_COLOR_GRAY;
        _flatView.headView.backgroundColor = FLAT_COLOR_GRAY;
        
        [self setChangeView];
        [self addDisconnectVC];
    }];
    
    [[PTDispatcher share] whenReadRSSI:^(NSNumber *RSSI) {
        NSIndexPath *index = [NSIndexPath indexPathForRow:0 inSection:0];
        UITableViewCell *tableCell = [self.tableView cellForRowAtIndexPath:index];
        
        NSArray *strengthImages = @[@"signal_strength_0",@"signal_strength_1",@"signal_strength_2",@"signal_strength_3",@"signal_strength_4",@"signal_strength_5"];
        
        NSNumber *strength = [PTPrinter strengthWithRSSI:RSSI];
        tableCell.imageView.image = [UIImage imageNamed:strengthImages[strength.integerValue]];
    }];
}

- (void)addDisconnectVC
{
    if ([PTDispatcher share].mode == PTDispatchModeUnconnect) {
        self.disconnectVC = [[PTDisconnectVC alloc] init];
        [self addChildViewController:self.disconnectVC];
        [self.view addSubview:self.disconnectVC.view];
        [self.disconnectVC didMoveToParentViewController:self];
    }
}

- (void)removeDisconnectVC
{
    [self.disconnectVC willMoveToParentViewController:nil];
    [self.disconnectVC.view removeFromSuperview];
    [self.disconnectVC removeFromParentViewController];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:YES];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationUserSelectCell:) name:@"NotificationUserSelectCell" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(startShimmering) name:UIApplicationDidBecomeActiveNotification object:nil];
}

- (void)viewWillDisappear:(BOOL)animated {
    
    [super viewWillDisappear:YES];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self removeDisconnectVC];
}

#pragma mark Setup View

/** 判断是否连接打印机，改变视图展示信息 */
- (void)setChangeView {
    
    [self removeDisconnectVC];
    
    UIColor *headColor;
    
    switch ([PTDispatcher share].mode) {
            
        case PTDispatchModeUnconnect:
            
            headColor = FLAT_COLOR_GRAY;
            break;
        case PTDispatchModeBLE:
            
            headColor = FLAT_COLOR_BLUE;
            _currentPrinterName = [PTDispatcher share].printerConnected.name;
            break;
            
        case PTDispatchModeWiFi:
            
            headColor = FLAT_COLOR_BLUE;
            _currentPrinterName = [PTDispatcher share].printerConnected.name;
            break;
            
        default:
            break;
    }
    
    self.navigationController.navigationBar.barTintColor = headColor;
    _flatView.headView.backgroundColor = headColor;
    
    [_tableView reloadData];
}

#pragma mark - init Settings

- (void)initSettings {
   
    NSString *color = [[PTSettings getUserDefaultObjectByKey:kThemeColor] uppercaseString];
    _themeColor = [[NSString alloc] initWithFormat:@"Flat_COLOR_%@",color];
}

#pragma mark - init DataSource

/** 初始化屏幕适配的相关参数 */
- (void)initScreentFitDataSource {
    
    _buttonWidth = [FlatFit getMainButtonWidth];
    _buttonHeight = [FlatFit getMainButtonHeight];
}

/** 手动切换指令集 */
- (void)setupCommand:(UISegmentedControl *)sender {
    
    _selectedSegmentIndex = sender.selectedSegmentIndex;
    
    switch (_selectedSegmentIndex) {
        case 0:  [self initDataSourceESC];  break;
            
        default:
            break;
    }
    
    [_tableView reloadData];
}

- (void)initDataSourceESC {
    
    [PTDispatcher share].printerConnected.commandMode = PTCommandModeESC;
    _titleArray = @[@[@"Wireless Printer"],
                    @[@"Text",@"Receipt",@"Barcode"],
                    @[@"Barcode",@"Text",@"Page Mode"],
                    @[@"Setting"]];
    
    _detailArray = @[@[@"Current connected printer"],
                     @[@"Text_Detial_Title",@"Receipt_Detial_Title",@"Barcode_Detial_Title"],
                     @[],
                     @[]];
    
    _iconArray = @[@[@""],@[icon_format,icon_receipt,icon_qrcode],@[icon_barcode,icon_text,icon_document],@[icon_setting]];
}

#pragma mark - init View

- (CGFloat)getSplitPercentage {
    
    NSInteger resolution = SCREEN_RESOLUTION;
    CGFloat splitPercenTage;
    switch (resolution) {
            
        case iPhone4:   splitPercenTage = 0.23; break;
        case iPhone5:   splitPercenTage = 0.2;  break;
        case iPhone6:   splitPercenTage = 0.17; break;
        case iPhone6p:  splitPercenTage = 0.15; break;
        case iPad:      splitPercenTage = 0.12; break;
        default:        splitPercenTage = 0.12; break;
    }
    return splitPercenTage;
}

- (void)initFlatMainView {
    
    UIColor *headColor = FLAT_COLOR_GRAY;
    UIColor *footColor = [UIColor whiteColor];
    
    _flatView = [[FlatMainView alloc] initWithWidth:SCREEN_WIDTH Height:SCREEN_HEIGHT splitPercentage:[self getSplitPercentage] headColor:headColor footColor:footColor];
    [self.view addSubview:_flatView];
}

- (void)initTitleLabel {
    
    _titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0 , 120, 44)];
    _titleLabel.backgroundColor = [UIColor clearColor];
    _titleLabel.font = [UIFont fontWithName:@"Gurmukhi MN" size:25];
    _titleLabel.textColor = [UIColor whiteColor];//设置文本颜色
    _titleLabel.textAlignment = NSTextAlignmentCenter;
    _titleLabel.text = @"Wireless Printer";
    self.navigationItem.titleView = _titleLabel;
}

- (void)initSegment {
    
    NSArray *segmentTitles = @[@"ESC"];
    UISegmentedControl *segment = [[UISegmentedControl alloc] initWithItems:segmentTitles];
    segment.tintColor = [UIColor whiteColor];
    segment.selectedSegmentIndex = 0;
    [segment addTarget:self action:@selector(setupCommand:) forControlEvents:UIControlEventValueChanged];
    [_flatView.headView addSubview:segment];
    
    CGSize segmentSize;
    if (SCREEN_RESOLUTION == iPad) {
        segmentSize = CGSizeMake(70 * segmentTitles.count, 30);
    }else {
        segmentSize = CGSizeMake(50 * segmentTitles.count, 25);
    }
    [segment mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(_flatView.headView).offset(-15);
        make.centerX.equalTo(_flatView.headView);
        make.size.mas_equalTo(segmentSize);
    }];
}

- (void)initTableView {
    
    CGRect tableViewRect = CGRectMake(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT*(1-[self getSplitPercentage])-TabBar_Height);
    _tableView = [[UITableView alloc] initWithFrame:tableViewRect style:UITableViewStyleGrouped];
    _tableView.dataSource = self;
    _tableView.delegate = self;
    _tableView.scrollEnabled = YES;
    _tableView.tableHeaderView = [[UIView alloc] init];
    _tableView.tableFooterView = [[UIView alloc] init];
    
    [_flatView.footView addSubview:_tableView];
    
    UIView *v = [[UIView alloc] initWithFrame:CGRectZero];//隐藏多余的 cell
    [_tableView setTableFooterView:v];
}

#pragma mark - UITableViewDelegate

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{

    return [FlatFit rowHeight];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    
    return 30;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section {
    
    return 1;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView;
{
    return _titleArray.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [_titleArray[section] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *iden = @"cellIden";
    UITableViewCell *tableCell;
    
    if ((indexPath.section == 0) && (indexPath.row == 0)) {
        
        tableCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"commandVCCellIden"];
    }else {
        
        tableCell = [tableView dequeueReusableCellWithIdentifier:iden];
        if (tableCell == nil) {
            tableCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:iden];
        }
    }
    
    tableCell.accessoryType = UITableViewCellAccessoryNone;
    tableCell.textLabel.text = @"";
    tableCell.detailTextLabel.text = @"";
    
    if ([_titleArray[0] count] != 0) { //防止 cell 崩溃
        
        UIColor *color = [UIColor colorWithRed:0.000 green:0.615 blue:1.000 alpha:1.000];
        PTSymbolFont *font = [[PTSymbolFont alloc] initWithCode:_iconArray[indexPath.section][indexPath.row] size:28 fontName:@"WirelessPrinter" color:color];
        tableCell.imageView.image = [font image:CGSizeMake(28, 30)];
        tableCell.imageView.contentMode = UIViewContentModeScaleAspectFit;
        
        NSString *textLocal = NSLocalizedString(_titleArray[indexPath.section][indexPath.row], nil);
        tableCell.textLabel.text = textLocal;
        
        if ([_detailArray[indexPath.section] count] != 0) {
            
            NSString *detailTextLocal = NSLocalizedString(_detailArray[indexPath.section][indexPath.row], nil);
            tableCell.detailTextLabel.text = detailTextLocal;
        }
        
        if (indexPath.section == 0) {
            
            
        }else if (indexPath.section == 2) {
            
            tableCell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        }else if (indexPath.section == 3){
            
            NSString *version = [FlatToolBox getCurrentVersion];
            tableCell.detailTextLabel.text = version;
        }
        
        if ((indexPath.section == 0) && (indexPath.row == 0)) {
            tableCell.textLabel.text = _currentPrinterName;
            NSString *detailTextLocal = NSLocalizedString(_detailArray[indexPath.section][indexPath.row], nil);
            tableCell.detailTextLabel.text = detailTextLocal;
            
            float tagLabelWidth = [FlatFit rowHeight]/0.65;
            float tagLabelHeight = [FlatFit rowHeight];
            
            UIButton *disconnectButton = [[UIButton alloc] init];
            disconnectButton.frame = CGRectMake(SCREEN_WIDTH-tagLabelWidth, 0, tagLabelWidth, tagLabelHeight);
            disconnectButton.layer.borderWidth = 0.5;
            disconnectButton.layer.borderColor = [UIColor grayColor].CGColor;
            [disconnectButton setTitle:NSLocalizedString(@"OFF", nil) forState:UIControlStateNormal];
            disconnectButton.titleLabel.font = [UIFont systemFontOfSize:[FlatFit titleFontSize]];
            [disconnectButton setTitleColor:[UIColor grayColor] forState:UIControlStateNormal];
            [disconnectButton setTitleColor:FLAT_COLOR_ORANGE forState:UIControlStateHighlighted];
            [disconnectButton addTarget:self action:@selector(disConnectCurrentLine) forControlEvents:UIControlEventTouchUpInside];
            [tableCell addSubview:disconnectButton];
        }
        
        tableCell.detailTextLabel.textColor = [UIColor grayColor];
    }
    
    return tableCell;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section;
{
    NSArray *sectionTitle = @[@"Printer",@"Display",@"Func",@"About"];
    if (self.selectedSegmentIndex == 1) {
        sectionTitle = @[@"Printer",@"Test",@"Func",@"About"];
    }
    return NSLocalizedString(sectionTitle[section], nil);
}

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];//取消选中 cell
    
    switch (_selectedSegmentIndex) {
        case 0: [self escPosFunctionSwitcher:indexPath];   break;
            
        default:        break;
    }
    
}

- (void)escPosFunctionSwitcher:(NSIndexPath *)indexPath {
    
    PTTestESC *testESC = [[PTTestESC alloc] init];
    
    switch (indexPath.section) {
        case 1:
            switch (indexPath.row) {
//                case 0: [testESC testPrintMaxByte:5000];       break;
                case 0: [testESC printText];       break;
                case 1: [testESC printReceipt];    break;
                case 2: [testESC printBarcode];    break;
                default:                           break;
            }
            
            break;
        case 2:
            
            switch (indexPath.row) {
                case 0: [self popBarcodeVC];    break;
                case 1: [self popTextVC];       break;
                case 2: [self popPageVC];       break;
                default:                        break;
            }
            
            break;
        default:
            break;
    }
}

#pragma mark - Touch Response


- (void)popTextVC {
    
    PTTextVC *textVC = [[PTTextVC alloc] init];
    textVC.hidesBottomBarWhenPushed = YES;
    [self.navigationController pushViewController:textVC animated:YES];
}

- (void)popBarcodeVC {
    
    PTBarcodeVC *barcodeVC = [[PTBarcodeVC alloc] init];
    barcodeVC.hidesBottomBarWhenPushed = YES;
    [self.navigationController pushViewController:barcodeVC animated:YES];
}

- (void)popPageVC {
    
    PTPageVC *pageVC = [[PTPageVC alloc] init];
    pageVC.hidesBottomBarWhenPushed = YES;
    [self.navigationController pushViewController:pageVC animated:YES];
}

- (void)popConnectVC {
    
    if ([PTDispatcher share].mode == PTDispatchModeUnconnect) {
        
        PTConnectVC *connectVC = [[PTConnectVC alloc] init];
        [self presentViewController:connectVC animated:YES completion:^{
            
        }];
        
    }else {
        
        [[PTDispatcher share] disConnectPrinter];
    }
}

#pragma mark - Notification Response

- (void)startShimmering {
    
    [_titleLabel startShimmering];
}

- (void)notificationUserSelectCell:(NSNotification *)sender {
    
    _tableRowDic = [sender userInfo];
}

#pragma mark - Private Method

- (void)empty {
    
    _tableRowDic = nil;
}

- (void)disConnectCurrentLine {
    
    [[PTDispatcher share] disConnectPrinter];
}

- (void)didReceiveMemoryWarning {
    
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
