//
//  PTFlashConnectVC.swift
//  WirelessPrinter
//
//  Created by midmirror on 16/7/12.
//  Copyright © 2016年 midmirror. All rights reserved.
//

import Foundation
import UIKit
import CoreBluetooth

/** 蓝牙闪联：自动连接 0.2 m 范围内的蓝牙 */
open class PTFlashConnectVC: UIViewController,UITableViewDataSource,UITableViewDelegate {
    
    var pulseView: UIView?
    var phoneLogoView: UIImageView?
    var printer: PTPrinter?
    var tableView = UITableView.init(frame: CGRect(x: 0, y: ScreenHeight*0.6, width: ScreenWidth, height: ScreenHeight*0.25), style: UITableViewStyle.plain)
    var pulsetimer: Timer?
    
    override open func viewDidLoad() {
        super.viewDidLoad()
        self.view.backgroundColor = UIColor.white
        
        let buttonHeight = FlatFit.getMainButtonHeight()
        let closeButton = UIButton()
        closeButton.frame = CGRect(x: ScreenWidth/4, y: ScreenHeight*0.9, width: ScreenWidth/2, height: buttonHeight)
        closeButton.addTarget(self, action: #selector(closeView), for: UIControlEvents.touchUpInside)
        
        closeButton.layer.cornerRadius = buttonHeight/2
        closeButton.backgroundColor = flatColorGray
        closeButton.setTitle(NSLocalizedString("Cancel", comment: ""), for: UIControlState())
        self.view.addSubview(closeButton)
        
        self.rangeStepper.value = PTSetting.share().getScanDistance()
        self.setNotifySwitch.isOn = PTSetting.share().getScanSetting(withKey: setting_receive_message)
        self.setFeedSwitch.isOn = PTSetting.share().getScanSetting(withKey: setting_feed_one_line)
        
        self.tableView.dataSource = self
        self.tableView.delegate = self
        self.tableView.isScrollEnabled = false
        self.tableView.tableFooterView = UIView()
        self.view.addSubview(self.tableView)
        
//        unowned let weakSelf = self;
//        PTPrinter.share().scanBluetooth { (peripheral, advertisementData, RSSI) in
//            
//            let distance = PTPrinter.share().generateDistanceWithRSSI(RSSI)
//            print("name:\(peripheral.name) distance:\(distance)")
//            
//            if distance.doubleValue < self.rangeStepper.value {
//                
//                weakSelf.peripheral = peripheral
//                PTPrinter.share().stopScanBluetooth()
//                PTPrinter.share().connectBluetooth(weakSelf.peripheral, notify: weakSelf.setNotifySwitch.on, success: { (peripheral) in
//                    weakSelf.closeView()
//                    }, failure: { (peripheral) in
//                        
//                    }, canSendData: { (characteristic) in
//                        
//                })
//            }
//        }
    }
    
    let alertLabel = UILabel()
    let deviceLabel = UILabel()
    let headView = UIView.init(frame: CGRect(x: 0, y: 0, width: ScreenWidth, height: ScreenHeight*0.6))
    
    override open func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        headView.backgroundColor = flatColorGray
        self.view.addSubview(headView)
        
        deviceLabel.frame = CGRect(x: 0, y: headView.frame.size.height-65, width: ScreenWidth, height: 25)
        deviceLabel.font = UIFont.systemFont(ofSize: FlatFit.titleFontSize()-2)
        deviceLabel.textAlignment = NSTextAlignment.center
        deviceLabel.textColor = UIColor.white
        self.view.addSubview(deviceLabel)
        
        alertLabel.frame = CGRect(x: 0, y: headView.frame.size.height-40, width: ScreenWidth, height: 30);
        alertLabel.text = NSLocalizedString("please move your phone near to printer", comment: "")
        alertLabel.font = UIFont.systemFont(ofSize: FlatFit.titleFontSize()-2)
        alertLabel.textAlignment = NSTextAlignment.center
        alertLabel.textColor = UIColor.white
        self.view.addSubview(alertLabel)
        
        let screenWidth = UIScreen.main.bounds.size.width
        
        self.pulseView = UIView.init(frame: CGRect(x: screenWidth*0.1, y: (self.tableView.frame.origin.y-screenWidth*0.8)/2, width: screenWidth*0.8, height: screenWidth*0.8))
        self.view.addSubview(self.pulseView!)
        
        self.phoneLogoView = UIImageView()
        self.phoneLogoView!.frame = CGRect(x: (self.pulseView!.frame.size.width-30)/2, y: (self.pulseView!.frame.size.height-60)/2, width: 30, height: 60)
        self.phoneLogoView!.image = UIImage.init(named: "iphone")
        self.pulseView!.addSubview(self.phoneLogoView!)
        
        self.pulsetimer = Timer.init(timeInterval: 1, target: self, selector: #selector(startAnimation), userInfo: nil, repeats: true)
        RunLoop.main.add(self.pulsetimer!, forMode: RunLoopMode.commonModes)
        
        PTDispatcher.share().scanBluetooth { (printer) in
            self.printer = printer
            if self.printer!.distance.floatValue < 1.0 {
                self.deviceLabel.text = String.init(format: "\(self.printer!.name!): %.2f m", (self.printer?.distance.floatValue)!)
            }
            
            if self.printer!.distance.doubleValue < self.rangeStepper.value {
                
                PTDispatcher.share().stopScanBluetooth()
                
                if self.pulsetimer != nil {
                    self.pulsetimer!.invalidate()
                    self.pulsetimer = nil
                }
                
                
                self.headView.backgroundColor = flatColorBlue
                
                self.alertLabel.text = NSLocalizedString("Connecting...", comment: "")
                ProgressHUD.show("\(self.printer!.name!)")
                
                PTDispatcher.share().connect(self.printer)
                
                PTDispatcher.share().whenConnectSuccess({
                    ProgressHUD.showSuccess("Success!")
                    PTPrinterPort.saveBluetooth(withUUID: self.printer!.uuid, name: self.printer!.name)
                    self.closeView()
                    
                    if self.setFeedSwitch.isOn == true {
                        let carriageReturn: [UInt8] = [0x0a];
                        let data = Data.init(bytes: UnsafePointer<UInt8>(carriageReturn), count: 1);
                        PTDispatcher.share().send(data);
                    }
                    ProgressHUD.showSuccess("Printer Ready!")
                })
                
                PTDispatcher.share().whenConnectFailure({
                    ProgressHUD.showError("Failure!")
                })
            }
        }
    }
    
    override open func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
    }

    func startAnimation() {
        
        let effect  = FlatEffectLayer()
        let pulseLayer = effect.generatePulseLayer(self.pulseView!)
        self.pulseView!.layer.insertSublayer(pulseLayer!, below: nil)
        self.perform(#selector(removeLayer(_:)), with: pulseLayer, afterDelay: 2)
    }
    
    func removeLayer(_ layer: CALayer) {
        
        layer.removeFromSuperlayer()
    }
    
    func closeView() {
        
        PTSetting.share().saveScanSetting(withKey: setting_receive_message, value: self.setNotifySwitch.isOn)
        PTSetting.share().saveScanSetting(withKey: setting_feed_one_line, value: self.setFeedSwitch.isOn)
        PTSetting.share().saveScanDistance(self.rangeStepper.value)
        PTSetting.share().sychronize()
        PTDispatcher.share().stopScanBluetooth()
        self.dismiss(animated: true) { 
            
        }
    }
    
    var tickTime = Date()
    
    // MARK: - UITableViewDataSource
    
    open func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        
        return 3;
    }
    
    let setNotifySwitch =  UISwitch.init(frame: CGRect(x: 0, y: 0, width: 44, height: 44))
    let setFeedSwitch =  UISwitch.init(frame: CGRect(x: 0, y: 0, width: 44, height: 44))
    let rangeStepper = UIStepper.init(frame: CGRect(x: 0, y: 0, width: 100, height: 44))
    var bluetoothRange = "Reaction Range: \(PTSetting.share().getScanDistance()) m"

    open func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cellIdentifier = "PTReaderListVC"
        var cell = tableView.dequeueReusableCell(withIdentifier: cellIdentifier)
        
        if cell == nil {
            
            cell = UITableViewCell.init(style: UITableViewCellStyle.subtitle, reuseIdentifier: cellIdentifier)
        }
        
        if (indexPath as NSIndexPath).row == 0 {
            
            cell!.textLabel!.text = NSLocalizedString("Enable Receive Printer Message", comment: "");
            cell!.accessoryView = self.setNotifySwitch;
        } else if (indexPath as NSIndexPath).row == 1 {
            
            cell!.textLabel!.text = NSLocalizedString("Enable Printer to React when Connected", comment: "");
            cell!.accessoryView = self.setFeedSwitch;
        } else if (indexPath as NSIndexPath).row == 2 {
            
            self.bluetoothRange = "\(NSLocalizedString("Reaction Range:", comment: "")) \(self.rangeStepper.value) m"
            cell!.textLabel!.text = self.bluetoothRange;
            self.rangeStepper.wraps = true;
            self.rangeStepper.minimumValue = 0.1;
            self.rangeStepper.maximumValue = 1;
            self.rangeStepper.stepValue = 0.1;
            self.rangeStepper.addTarget(self, action: #selector(reloadTableData), for: UIControlEvents.touchUpInside)
            cell!.accessoryView = self.rangeStepper;
        }
        
        return cell!
    }
    
    open func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        tableView.deselectRow(at: indexPath, animated: true)
    }
    
    func reloadTableData() {
    
        self.tableView.reloadData()
    }
}
