//
//  PTBarcodeVC.m
//
//  Created by midmirror on 15/12/25.
//  Copyright © 2015年 midmirror. All rights reserved.
//

#import "PTBarcodeVC.h"
#import "Masonry.h"
#import "PTPrinterPort.h"

@interface PTBarcodeVC ()

@property(strong,nonatomic,readwrite) NSDictionary *tableRowDic;
@end

@implementation PTBarcodeVC

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    UILabel *titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0 , 200, 44)];
    titleLabel.backgroundColor = [UIColor clearColor];
    titleLabel.font = [UIFont systemFontOfSize:20];
    titleLabel.textColor = [UIColor whiteColor];//设置文本颜色
    titleLabel.textAlignment = NSTextAlignmentCenter;
    titleLabel.text = @"QRCode/BarCode";
    self.navigationItem.titleView = titleLabel;
    
    [self setupDataSrouce];
    [self setupFlatUI];
    [self setupFootTable];
}

- (void)viewDidAppear:(BOOL)animated {
    
    [super viewDidAppear:YES];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationUserSelectCell:) name:@"NotificationUserSelectCell" object:nil];
}

- (void)notificationUserSelectCell:(NSNotification *)sender {
    
    _tableRowDic = [sender userInfo];
    int row = [[_tableRowDic objectForKey:@"row"] intValue];
    [self popPickerViewAt:row];
}

- (void)setupFlatUI {
    
    self.navigationController.navigationBar.barTintColor = FLAT_COLOR_BLUE;
    [self.navigationController.navigationBar setTranslucent:NO];
    
    UIColor *headColor = FLAT_COLOR_BLUE;
    UIColor *footColor = FLAT_COLOR_COULD;
    
    _flatView = [[FlatMainView alloc] initWithWidth:SCREEN_WIDTH Height:SCREEN_HEIGHT splitPercentage:0.4 headColor:headColor footColor:footColor];
    _flatView.frame = CGRectMake(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
    [self.view addSubview:_flatView];
    
    _inputTF = [[UITextField alloc] init];
    _inputTF.borderStyle = UITextBorderStyleRoundedRect;
    _inputTF.placeholder = @"QRCode/BarCode";
    _inputTF.text = @"075678164125";
    [_flatView.headView addSubview:_inputTF];
    
    [_inputTF mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view.mas_centerX);
        make.top.equalTo(self.view.mas_top).offset(SCREEN_HEIGHT*0.04);
        make.size.mas_equalTo(CGSizeMake(SCREEN_WIDTH * 0.8, SCREEN_WIDTH * 0.1));
    }];
    
    _flatView.mainButton.hidden = NO;
    [_flatView.mainButton setTitle:NSLocalizedString(@"Send To Print", nil) forState:UIControlStateNormal];
    [_flatView.mainButton addTarget:self action:@selector(printAction) forControlEvents:UIControlEventTouchUpInside];
    
    CGFloat buttonWidth = [FlatFit getMainButtonWidth];
    CGFloat buttonHeight = [FlatFit getMainButtonHeight];
    
    [_flatView.mainButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.bottom.equalTo(_flatView.headView.mas_bottom).offset(-SCREEN_HEIGHT*0.04);
        make.size.mas_equalTo(CGSizeMake(buttonWidth, buttonHeight));
    }];
}

- (void)setupDataSrouce {
    
    self.barcodeTypeDict = @{@"UPC-A":@"0",@"UPC-E":@"1",@"EAN8":@"3",@"EAN13":@"2",@"CODE39":@"4",@"ITF":@"5",@"CODEBAR":@"6",@"CODE128":@"73",@"CODE93":@"72",@"QRCODE":@"QRCODE"};
    
    NSArray *alignArray = @[@"Left",@"Center",@"right"];
    NSArray *numberAroundArray = @[@"None",@"print up",@"print down",@"up and down"];
    _detailListArray = @[@[self.barcodeTypeDict.allKeys],@[alignArray],@[numberAroundArray],@6,@80];
    
    _codeNumWithTypeDic = @{@"UPC-A":@"075678164125",@"UPC-E":@"01227000009",@"EAN8":@"04210009",@"EAN13":@"6901028075831",@"CODE39":@"123456789",@"ITF":@"123456789012",@"CODEBAR":@"AA12584A",@"CODE128":@"A1256",@"CODE93":@"TEST93",@"QRCODE":@"123456789"};
    
    _detailTextArray = [[NSMutableArray alloc] initWithObjects:@"UPC-A",@"Left",@"None",@"2",@"80", nil];
    _parameterArray = [[NSMutableArray alloc] initWithObjects:@0,@0,@0,@2,@80, nil];
}


- (void)popPickerViewAt:(int)row {
    
    UIColor *headColor = FLAT_COLOR_GRAY;
    
    _popView = [[FlatPopView alloc] initWithWidth:SCREEN_WIDTH*0.6 Height:SCREEN_HEIGHT*0.4 splitPercentage:0.2 headColor:headColor];
    _popView.titleLabel.text = NSLocalizedString(@"Please Select", nil);
    __block FlatPopView *popView = _popView;
    __block PTBarcodeVC *barcodeVC = self;
    [_popView addButtonWithTitle:NSLocalizedString(@"OK", nil) handler:^(FlatPopViewButton *button) {
        
        [popView hideAnimation];        // 隐藏弹出视图
        [barcodeVC updateTitleAndParameter:[NSNumber numberWithInt:row]];   // 更新 FootView 上表视图的副标题，和打印参数
        [barcodeVC setupFootTable];    // 更新 footView 上的表视图
    }];
    
    [_popView show];
    
    NSArray *dataArray;
    if (row < 3 ) {
        
        dataArray = _detailListArray[row];
    }else {
        
        FlatToolBox *tool = [[FlatToolBox alloc] init];
        dataArray = [tool generateNumberArrayWith:_detailListArray[row]];
    }
    
    FlatPickerKit *picker = [[FlatPickerKit alloc] initWithWidth:SCREEN_WIDTH*0.6 Height:SCREEN_HEIGHT*0.4*0.8 withDataSource:dataArray];
    [_popView.footView addSubview:picker];
}

- (void)updateTitleAndParameter:(NSNumber *)row {
    
    if (row) {
        
        int rowInt = [row intValue];
        FlatToolBox *tool = [[FlatToolBox alloc] init];
        NSString *newDetailTitle = [tool pieceStringWithArray:[FlatPickerKitModel share].titleArray];
        if (rowInt > 2) {
            newDetailTitle = [NSString stringWithFormat:@"%d",[newDetailTitle intValue]];
            [_parameterArray replaceObjectAtIndex:rowInt withObject:newDetailTitle];
            
        }else {
            NSString *pickerRow = [[FlatPickerKitModel share].rowArray[0] stringValue];
            
            if (rowInt == 0) {
                _inputTF.text = [_codeNumWithTypeDic objectForKey:newDetailTitle];
            }
           [_parameterArray replaceObjectAtIndex:rowInt withObject:pickerRow];
        }
        [_detailTextArray replaceObjectAtIndex:[row integerValue] withObject:newDetailTitle];
    }
}

- (void)setupFootTable {
    
    NSArray *titleArray = @[NSLocalizedString(@"Type", nil),
                            NSLocalizedString(@"Justification", nil),
                            NSLocalizedString(@"Position", nil),
                            NSLocalizedString(@"Width", nil),
                            NSLocalizedString(@"Height", nil)];
    NSArray *tableDataArray = @[titleArray,_detailTextArray];
    
    _tableKit = [[FlatTableKit alloc] initWithWidth:SCREEN_WIDTH Height:SCREEN_HEIGHT*0.6 DataArray:tableDataArray TableMode:FlatTableModeNormal CheckMark:NO];
    
    [_flatView.footView addSubview:_tableKit];
}

- (void)printAction
{
    NSString *text = _inputTF.text;
    
    NSInteger type            = [self.barcodeTypeDict[_detailTextArray[0]] integerValue];
    NSInteger justification   = [_parameterArray[1] intValue];
    NSInteger hri             = [_parameterArray[2] intValue];
    NSInteger width           = [_parameterArray[3] intValue];
    NSInteger height          = [_parameterArray[4] intValue];
    
    PTCommandESC *cmd = [[PTCommandESC alloc] init];
    if (type == 9) {
        
        [cmd appendQRCodeData:text justification:justification leftMargin:0 eccLevel:48 model:50 size:26];
    }else {
        
        [cmd appendBarcode:type data:text justification:justification width:width height:height hri:hri];
    }
    
    [PTPrinterPort sendData:[cmd getCommandData]];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillDisappear:(BOOL)animated {
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [super viewWillDisappear:YES];
}

@end
