//
//  PTTextVC.m
//  BluetoothPrinter
//
//  Created by midmirror on 15/12/28.
//  Copyright © 2015年 midmirror. All rights reserved.
//

#import "PTTextVC.h"
#import "Masonry.h"
#import "PTPrinterPort.h"

@interface PTTextVC ()

@property(strong,nonatomic,readwrite) NSDictionary *tableRowDic;
@end

@implementation PTTextVC

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    UILabel *titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0 , 200, 44)];
    titleLabel.backgroundColor = [UIColor clearColor];
    titleLabel.font = [UIFont systemFontOfSize:20];
    titleLabel.textColor = [UIColor whiteColor];//设置文本颜色
    titleLabel.textAlignment = NSTextAlignmentCenter;
    titleLabel.text = @"Text Print";
    self.navigationItem.titleView = titleLabel;
    
    [self setupDataSource];
    [self setupFlatView];
    [self setupFootTable];
    
    _tableKit = [FlatTableKit share];
    
}

/** 通知：获取用户点击表视图组件的所在行 */
- (void)notificationUserSelectCell:(NSNotification *)sender {
    
    _tableRowDic = [sender userInfo];
    int row = [[_tableRowDic objectForKey:@"row"] intValue];
    
    if (row >= 6) {
        
        [self popPickerViewAt:row];
    }else {
        
        [self updateTitleAndParameter:[NSNumber numberWithInt:row]];
        [self setupFootTable];
    }
}

- (void)viewWillAppear:(BOOL)animated {
    
    self.navigationController.navigationBar.barTintColor = FLAT_COLOR_BLUE;
    [self.navigationController.navigationBar setTranslucent:NO];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationUserSelectCell:) name:@"NotificationUserSelectCell" object:nil];
    [super viewWillAppear:YES];
}

- (void)viewWillDisappear:(BOOL)animated {
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [super viewWillDisappear:YES];
}

/** 设置数据源 */
- (void)setupDataSource {
    
    _detailTextArray = [[NSMutableArray alloc] initWithObjects:@"NO",@"NO",@"NO",@"NO",@"NO",@"NO",@"0",@"0",@"0",@"Left", nil];
    _parameterArray = [[NSMutableArray alloc] initWithObjects:@"NO",@"NO",@"NO",@"NO",@"NO",@"NO",@"0",@"0",@"0",@"0", nil];
}

/** 设置主界面 */
- (void)setupFlatView {
    
    UIColor *headColor = FLAT_COLOR_BLUE;
    UIColor *footColor = FLAT_COLOR_COULD;
    
    _flatView = [[FlatMainView alloc] initWithWidth:SCREEN_WIDTH Height:SCREEN_HEIGHT splitPercentage:0.4 headColor:headColor footColor:footColor];
    [self.view addSubview:_flatView];
    
    [_flatView.textView setText:@"Xiamen Hanin Electronic Technology Co., Ltd."];
    
    [_flatView.textView mas_makeConstraints:^(MASConstraintMaker *make) {
        
        make.centerX.equalTo(self.view.mas_centerX);
        make.top.equalTo(self.view.mas_top).offset(SCREEN_HEIGHT*0.03);
        make.size.mas_equalTo(CGSizeMake(SCREEN_WIDTH * 0.8, SCREEN_WIDTH * 0.15));
    }];
    
    _flatView.mainButton.hidden = NO;
    [_flatView.mainButton setTitle:NSLocalizedString(@"Send To Print", nil) forState:UIControlStateNormal];
    [_flatView.mainButton addTarget:self action:@selector(sendToPrinterAction) forControlEvents:UIControlEventTouchUpInside];
    
    CGFloat buttonWidth = [FlatFit getMainButtonWidth];
    CGFloat buttonHeight = [FlatFit getMainButtonHeight];
    
    [_flatView.mainButton mas_makeConstraints:^(MASConstraintMaker *make) {
        
        make.centerX.equalTo(self.view);
        make.bottom.equalTo(_flatView.headView.mas_bottom).offset(-SCREEN_HEIGHT*0.03);
        make.size.mas_equalTo(CGSizeMake(buttonWidth, buttonHeight));
    }];
}

/** 更新表视图 Cell 的标题和打印时的参数 */
- (void)updateTitleAndParameter:(NSNumber *)row {
    
    if (row) {
        
        int rowInt = [row intValue];
        if ([row intValue] > 5) {
            
            FlatToolBox *tool = [[FlatToolBox alloc] init];
            NSString *newDetailTitle = [tool pieceStringWithArray:[FlatPickerKitModel share].titleArray];
            
            if ([row intValue] != 9) {
                newDetailTitle = [NSString stringWithFormat:@"%d",[newDetailTitle intValue]];
                [_parameterArray replaceObjectAtIndex:rowInt withObject:newDetailTitle];
            }else {
                NSString *pickerRow = [[FlatPickerKitModel share].rowArray[0] stringValue];
                [_parameterArray replaceObjectAtIndex:rowInt withObject:pickerRow];
            }
            [_detailTextArray replaceObjectAtIndex:rowInt withObject:newDetailTitle];
            
        }else {
            
            if ([_detailTextArray[rowInt] isEqualToString:@"YES"]) {
                
                [_detailTextArray replaceObjectAtIndex:rowInt withObject:@"NO"];
                [_parameterArray replaceObjectAtIndex:rowInt withObject:@"NO"];
            }else {
                
                [_detailTextArray replaceObjectAtIndex:rowInt withObject:@"YES"];
                [_parameterArray replaceObjectAtIndex:rowInt withObject:@"YES"];
            }
        }
    }
}

/** 设置下方的表视图 */
- (void)setupFootTable {
    
    NSArray *titleArray = @[NSLocalizedString(@"Double Height", nil),
                            NSLocalizedString(@"Double Width", nil),
                            NSLocalizedString(@"Underline", nil),
                            NSLocalizedString(@"Bold", nil),
                            NSLocalizedString(@"Mini", nil),
                            NSLocalizedString(@"Reverse", nil),
                            NSLocalizedString(@"Double Height", nil),
                            NSLocalizedString(@"Double Width", nil),
                            NSLocalizedString(@"Left Margin", nil),
                            NSLocalizedString(@"Justification", nil)];
    
    NSArray *tableDataArray = @[titleArray,_detailTextArray];
    
    _tableKit = [[FlatTableKit alloc] initWithWidth:SCREEN_WIDTH Height:SCREEN_HEIGHT*0.6 DataArray:tableDataArray TableMode:FlatTableModeNormal CheckMark:NO];
    
    [_flatView.footView addSubview:_tableKit];
}

/** 在表视图的哪一行点击时弹出 PickerView */
- (void)popPickerViewAt:(int)row {
    
    UIColor *headColor = FLAT_COLOR_GRAY;
    
    _popView = [[FlatPopView alloc] initWithWidth:SCREEN_WIDTH*0.6 Height:SCREEN_HEIGHT*0.4 splitPercentage:0.2 headColor:headColor];
    _popView.titleLabel.text = NSLocalizedString(@"Please Select", nil);
    
    __block FlatPopView *popView = _popView;
    __block PTTextVC *textVC = self;
    [_popView addButtonWithTitle:NSLocalizedString(@"OK", nil) handler:^(FlatPopViewButton *button) {
        
        [popView hideAnimation];
        [textVC updateTitleAndParameter:[NSNumber numberWithInt:row]];
        [textVC setupFootTable];
    }];
    
    [_popView show];
    
    FlatToolBox *tool = [[FlatToolBox alloc] init];
    NSArray *dataArray;
    
    switch (row) {
            
        case 6: dataArray = [tool generateNumberArrayWith:@8];      break;
        case 7: dataArray = [tool generateNumberArrayWith:@8];      break;
        case 8: dataArray = [tool generateNumberArrayWith:@100];    break;
        case 9: dataArray = @[@[@"Left",@"Center",@"Right"]];       break;
            
        default:                                                    break;
    }
    
    FlatPickerKit *picker = [[FlatPickerKit alloc] initWithWidth:SCREEN_WIDTH*0.6 Height:SCREEN_HEIGHT*0.4*0.8 withDataSource:dataArray];
    [_popView.footView addSubview:picker];
}

/** 获取参数、发送打印 */
- (void)sendToPrinterAction {
    
    BOOL isDoubleWidth  = [_parameterArray[0] boolValue];
    BOOL isDoubleHeight = [_parameterArray[1] boolValue];
    BOOL isUnderLine    = [_parameterArray[2] boolValue];
    BOOL isBold         = [_parameterArray[3] boolValue];
    BOOL isMini         = [_parameterArray[4] boolValue];
    BOOL isReverse      = [_parameterArray[5] boolValue];
    
    int leftMargin      = [_parameterArray[8] intValue];
    int alignment       = [_parameterArray[9] intValue];
    
    
    if (0 <= leftMargin) {
        
        PTCommandESC *cmd = [[PTCommandESC alloc] init];
        
        [cmd setJustification:alignment];
        [cmd setLeftMargin:leftMargin];
        [cmd setTextStyleMini:isMini bold:isBold doubleWidth:isDoubleWidth doubleHeight:isDoubleHeight underline:isUnderLine];
        [cmd setReverse:isReverse];
        [cmd appendText:_flatView.textView.text];
        
        [PTPrinterPort sendData:[cmd getCommandData]];
        
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
