//
//  PTSymbolFont.swift
//  LabelDesigner
//
//  Created by midmirror on 16/7/5.
//  Copyright © 2016年 midmirror. All rights reserved.
//

import Foundation
import UIKit

open class PTSymbolFont: NSObject {
    
    var code: NSString = ""
    var fontName: String = ""
    var attributes = [String : AnyObject]()
    var color = UIColor()
    
    public init(code:NSString, size:CGFloat, fontName:String, color: UIColor) {
        self.code = code
        self.fontName = fontName
        self.attributes = [String : AnyObject]()
        self.attributes[NSParagraphStyleAttributeName] = NSMutableParagraphStyle.default.mutableCopy() as! NSMutableParagraphStyle
        
        // 使用代码注册字体、或者用 info.plist注册字体
        let font = UIFont.init(name: fontName, size: size)
        
        self.attributes[NSFontAttributeName] = font
        self.attributes[NSForegroundColorAttributeName] = color
        self.color = color
    }
    
    /** 将字体渲染成图片 */
    open func image(_ size: CGSize)->UIImage {
        
        UIGraphicsBeginImageContextWithOptions(size, false, 0)
        let context = UIGraphicsGetCurrentContext()
        
        let textRect  = CGRect(x: 0, y: 0, width: size.width, height: size.height)
        context?.setFillColor(self.color.cgColor);
        self.code.draw(in: textRect, withAttributes: self.attributes)
        
        let image = UIGraphicsGetImageFromCurrentImageContext();
        
        UIGraphicsEndImageContext()
        
        return image!
    }
    
    /** 返回一张正方形图片 */
    open func imageSquare(_ width: CGFloat) -> UIImage {
        
        return self.image(CGSize(width: width, height: width))
    }
}

/** 如果在 info.plist 中增加 Fonts provided by application这一项，并注册字体就不用这里的代码 */
open class PTFontLoader {
    
    static let shared = PTFontLoader()
    
    /** 载入字体文件 */
    public class func loadFont(fontName: String, size: CGFloat) -> UIFont {
        
        
        if (UIFont.fontNames(forFamilyName: fontName).count == 0) {
            
            registerFont(fontName)
        }
        
        return UIFont(name: fontName, size: size)!
    }
    
    /** 注册字体 */
    class func registerFont(_ fontName: String) {
        
        let fontPath = Bundle(for:object_getClass(self)).path(forResource: fontName, ofType:"ttf")
        
        let fileHandle : FileHandle = FileHandle(forReadingAtPath: fontPath!)!
        let data : Data = fileHandle.readDataToEndOfFile()
        
        let provider = CGDataProvider(data: data as CFData)
        let font = CGFont(provider!)
        
        var error: Unmanaged<CFError>?
        if !CTFontManagerRegisterGraphicsFont(font, &error) {
            
            let errorDescription: CFString = CFErrorCopyDescription(error!.takeUnretainedValue())
            let nsError = error!.takeUnretainedValue() as AnyObject as! NSError
            NSException(name: NSExceptionName.internalInconsistencyException, reason: errorDescription as String, userInfo: [NSUnderlyingErrorKey: nsError]).raise()
        }
    }
}

open class Icon {
    
    open static let NAME = "WirelessPrinter"
    open static let barcode  = "\u{e607}"
    open static let qrcode  = "\u{e606}"
    open static let printer = "\u{e605}"
    open static let image   = "\u{e602}"
    open static let document = "\u{e600}"
    open static let label   = "\u{e60f}"
    open static let receipt = "\u{e603}"
    open static let text    = "\u{e60b}"
    open static let format  = "\u{e604}"
    open static let label1  = "\u{e60f}"
    open static let label2  = "\u{e613}"
    open static let label3  = "\u{e60d}"
    
    open static let office = "\u{e601}"
    open static let ppt    = "\u{e608}"
    open static let doc    = "\u{e609}"
    open static let xls    = "\u{e60a}"
    open static let pdf    = "\u{e615}"
    
    open static let WiFi   = "\u{e618}"
    open static let BLE1    = "\u{e617}"
    open static let BLE2    = "\u{e619}"
    open static let BLE3    = "\u{e61A}"
    
    open static let chat   = "\u{e60c}"
    open static let setting = "\u{e612}"
    open static let setting2 = "\u{e61b}"
    open static let new    = "\u{e614}"
    open static let sent   = "\u{e611}"
    open static let switcher = "\u{e60e}"
}

